@@ -13,38 +13,40 @@
 # limitations under the License.
 
 import abc
-import apache_beam.metrics as metrics
 import logging
-import netCDF4 as nc
-import pygrib
 import shutil
 import tempfile
 import typing as t
-from apache_beam.io.filesystems import FileSystems
 from contextlib import contextmanager
 
-from .file_name_utils import OutFileInfo, GRIB_FILE_ENDINGS, NETCDF_FILE_ENDINGS
+import apache_beam.metrics as metrics
+import netCDF4 as nc
+import pygrib
+from apache_beam.io.filesystems import FileSystems
+
+from .file_name_utils import OutFileInfo
 
 logger = logging.getLogger(__name__)
 
 
 class SplitKey(t.NamedTuple):
-    level: str
-    short_name: str
+    levelType: str
+    shortname: str
 
     def __str__(self):
-        if not self.level:
-            return f'field {self.short_name}'
-        return f'{self.level} - field {self.short_name}'
+        if not self.levelType:
+            return f'field {self.shortname}'
+        return f'{self.levelType} - field {self.shortname}'
 
 
 class FileSplitter(abc.ABC):
     """Base class for weather file splitters."""
 
     def __init__(self, input_path: str, output_info: OutFileInfo,
-                 level: int = logging.INFO):
+                 force_split: bool = False, level: int = logging.INFO):
         self.input_path = input_path
         self.output_info = output_info
+        self.force_split = force_split
         self.logger = logging.getLogger(f'{__name__}.{type(self).__name__}')
         self.logger.setLevel(level)
         self.logger.debug('Splitter for path=%s, output base=%s',
@@ -52,7 +54,7 @@ class FileSplitter(abc.ABC):
 
     @abc.abstractmethod
     def split_data(self) -> None:
-        pass
+        raise NotImplementedError()
 
     @contextmanager
     def _copy_to_local_file(self) -> t.Iterator[t.IO]:
@@ -67,20 +69,35 @@ class FileSplitter(abc.ABC):
             shutil.copyfileobj(src_file, dest_file)
 
     def _get_output_file_path(self, key: SplitKey) -> str:
-        level = '{level}_'.format(level=key.level) if key.level else ''
-        return '{base}{level}{sn}{ending}'.format(
-            base=self.output_info.file_name_base, level=level, sn=key.short_name,
-            ending=self.output_info.ending)
+        split_keys = key._asdict()
+        if self.output_info.output_dir and key.levelType:
+            split_keys['levelType'] = f'{key.levelType}_'
+        return self.output_info.file_name_template.format(**split_keys)
+
+    def should_skip(self):
+        """Skip splitting if the data was already split."""
+        if self.force_split:
+            return False
+
+        for match in FileSystems().match([
+            self._get_output_file_path(SplitKey('', '**')),
+            self._get_output_file_path(SplitKey('**', '**')),
+        ]):
+            if len(match.metadata_list) > 0:
+                return True
+        return False
 
 
 class GribSplitter(FileSplitter):
 
-    def __init__(self, input_path: str, output_info: OutFileInfo):
-        super().__init__(input_path, output_info)
-
     def split_data(self) -> None:
         outputs = dict()
 
+        if self.should_skip():
+            metrics.Metrics.counter('file_splitters', 'skipped').inc()
+            self.logger.info('Skipping %s, file already split.', repr(self.input_path))
+            return
+
         with self._open_grib_locally() as grbs:
             for grb in grbs:
                 key = SplitKey(grb.typeOfLevel, grb.shortName)
@@ -106,10 +123,12 @@ class GribSplitter(FileSplitter):
 
 class NetCdfSplitter(FileSplitter):
 
-    def __init__(self, input_path: str, output_info: OutFileInfo):
-        super().__init__(input_path, output_info)
-
     def split_data(self) -> None:
+        if self.should_skip():
+            metrics.Metrics.counter('file_splitters', 'skipped').inc()
+            self.logger.info('Skipping %s, file already split.', repr(self.input_path))
+            return
+
         with self._open_dataset_locally() as nc_data:
             fields = [var for var in nc_data.variables.keys() if
                       var not in nc_data.dimensions.keys()]
@@ -150,27 +169,27 @@ class NetCdfSplitter(FileSplitter):
 
 
 class DrySplitter(FileSplitter):
-    def __init__(self, file_path: str, output_info: OutFileInfo):
-        super().__init__(file_path, output_info)
 
     def split_data(self) -> None:
         self.logger.info('input file: %s - output scheme: %s',
                          self.input_path, self._get_output_file_path(SplitKey('level', 'shortname')))
 
 
-def get_splitter(file_path: str, output_info: OutFileInfo,
-                 dry_run: bool) -> FileSplitter:
-    if output_info.ending in NETCDF_FILE_ENDINGS:
-        metrics.Metrics.counter('get_splitter', 'netcdf').inc()
-        if dry_run:
-            return DrySplitter(file_path, output_info)
-        return NetCdfSplitter(file_path, output_info)
-    if output_info.ending in GRIB_FILE_ENDINGS:
-        metrics.Metrics.counter('get_splitter', 'grib').inc()
-    else:
-        logger.info('unspecified file type, assuming grib for %s', file_path)
-        metrics.Metrics.counter('get_splitter',
-                                'unidentified grib').inc()
+def get_splitter(file_path: str, output_info: OutFileInfo, dry_run: bool, force_split: bool = False) -> FileSplitter:
     if dry_run:
         return DrySplitter(file_path, output_info)
-    return GribSplitter(file_path, output_info)
+
+    with FileSystems.open(file_path) as f:
+        header = f.read(4)
+
+    if b'GRIB' in header:
+        metrics.Metrics.counter('get_splitter', 'grib').inc()
+        return GribSplitter(file_path, output_info, force_split)
+
+    # See the NetCDF Spec docs:
+    # https://docs.unidata.ucar.edu/netcdf-c/current/faq.html#How-can-I-tell-which-format-a-netCDF-file-uses
+    if b'CDF' in header or b'HDF' in header:
+        metrics.Metrics.counter('get_splitter', 'netcdf').inc()
+        return NetCdfSplitter(file_path, output_info, force_split)
+
+    raise ValueError(f'cannot determine if file {file_path!r} is Grib or NetCDF.')
