@@ -39,7 +39,8 @@ def split_file(input_file: str,
                input_base_dir: str,
                output_template: t.Optional[str],
                output_dir: t.Optional[str],
-               dry_run: bool):
+               dry_run: bool,
+               force_split: bool = False):
     logger.info('Splitting file %s', input_file)
     metrics.Metrics.counter('pipeline', 'splitting file').inc()
     splitter = get_splitter(input_file,
@@ -47,7 +48,8 @@ def split_file(input_file: str,
                                                  input_base=input_base_dir,
                                                  output_template=output_template,
                                                  output_dir=output_dir),
-                            dry_run)
+                            dry_run,
+                            force_split)
     splitter.split_data()
 
 
@@ -63,7 +65,7 @@ def get_output_base_name(input_path: str,
                          input_base: str,
                          output_template: t.Optional[str],
                          output_dir: t.Optional[str]) -> OutFileInfo:
-    return get_output_file_base_name(input_path, output_template, output_dir, input_base)
+    return get_output_file_base_name(input_path, input_base, output_template, output_dir)
 
 
 def run(argv: t.List[str], save_main_session: bool = True):
@@ -80,9 +82,9 @@ def run(argv: t.List[str], save_main_session: bool = True):
             help='Template specifying path to output files using '
                  'python-style formatting substitution of input '
                  'directory names. '
-                 'For `input_pattern a/b/c/**` and file `a/b/c/file.nc`, '
-                 'a template with formatting `/somewhere/{1}-{0}.` '
-                 'will give `somewhere/c-file.shortname.nc`'
+                 'For `input_pattern a/b/c/**` and file `a/b/c/file.grib`, '
+                 'a template with formatting `/somewhere/{1}-{0}.{levelType}_{shortname}.grib` '
+                 'will give `somewhere/c-file.level_shortname.nc`'
                  )
     output_options.add_argument(
             '--output-dir', type=str,
@@ -90,10 +92,12 @@ def run(argv: t.List[str], save_main_session: bool = True):
                  'input_pattern. '
                  'For `input_pattern a/b/c/**` and file `a/b/c/file.nc`, '
                  '`output_template /x/y/z` will create '
-                 'output files like `/x/y/z/c/file_shortname.nc`'
+                 'output files like `/x/y/z/c/file.shortname.nc`'
                  )
     parser.add_argument('-d', '--dry-run', action='store_true', default=False,
                         help='Test the input file matching and the output file scheme without splitting.')
+    parser.add_argument('-f', '--force', action='store_true', default=False,
+                        help='Force re-splitting of the pipeline. Turns of skipping of already split data.')
     known_args, pipeline_args = parser.parse_known_args(argv[1:])
 
     configure_logger(2)  # 0 = error, 1 = warn, 2 = info, 3 = debug
@@ -104,6 +108,7 @@ def run(argv: t.List[str], save_main_session: bool = True):
     input_base_dir = _get_base_input_directory(input_pattern)
     output_template = known_args.output_template
     output_dir = known_args.output_dir
+
     if not output_template and not output_dir:
         raise ValueError('No output specified')
     dry_run = known_args.dry_run
@@ -126,5 +131,6 @@ def run(argv: t.List[str], save_main_session: bool = True):
                                            input_base_dir,
                                            output_template,
                                            output_dir,
-                                           dry_run)
+                                           dry_run,
+                                           known_args.force)
         )
